;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x3deea751e7e9b563fb9a9cc57ddc19f6 . "./code.cg2js/cg-acache.fasl")
(#x7db04dfa98273aa8b7517eab72c68a67 . "./code.cg2js/cg-base.fasl")
(#xfbd3fb06e3e95fd21c96d9729d6d3d19 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x4f9922425e174fbea4013ed3fa0f36a0 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x7044b4667bdb5a017e4f5250c99d2a49 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xc81317775b77b2827d128cad484c6a8c . "./code.cg2js/cg-button.fasl")
(#x8ff0366744a65a592809044cc3a01e9c . "./code.cg2js/cg-calendar.fasl")
(#xa4bc4dc436160b53c43f0e6d082f643b . "./code.cg2js/cg-caret.fasl")
(#xaa9ddd0430e8b8d57a0384beded7a0c6 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x0d0434e1a823dec66f6bc7a74a18a85b . "./code.cg2js/cg-chart-widget.fasl")
(#xdf6b2320657b90b6bbd787f95e97f5ae . "./code.cg2js/cg-check-box.fasl")
(#x27d9fa17a7640aed84c5e77baa92d31f . "./code.cg2js/cg-choice-list.fasl")
(#x0dd0f2801821a53b685a0b22f70890ab . "./code.cg2js/cg-choose-printer.fasl")
(#x48b312ebeaed8b1d4bbc9087bdf738f9 . "./code.cg2js/cg-class-grid.fasl")
(#x8ed14ae33e26cf0985740c93f168757b . "./code.cg2js/cg-class-slot-grid.fasl")
(#x5472bd63363881c888837a3dfa73664f . "./code.cg2js/cg-class-support.fasl")
(#xdd47e491121c68d9ba42ddb73cf8d87d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xfcf2cb602a1692530a1912ce4e1d5310 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x34aa6586935b9975d70dfb00686e158f . "./code.cg2js/cg-clipboard.fasl")
(#xa3e34837e6d89c4883291d45a1552fec . "./code.cg2js/cg-color-dialog.fasl")
(#x669cd5e46aa78f29d1fe1482d4274690 . "./code.cg2js/cg-combo-box.fasl")
(#xe2531681894569c56f3b29372adf26a7 . "./code.cg2js/cg-common-control.fasl")
(#xe0a426cf650ec11b1c658121ff3018ee . "./code.cg2js/cg-comtab.fasl")
(#x13cc0918b55bd2db36ae81b3ec0aa810 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x6177c7042181c8099aad7019e0206534 . "./code.cg2js/cg-curve.fasl")
(#x8a70b49d95039d3398dbc69ba1b17aef . "./code.cg2js/cg-dde-utils.fasl")
(#xded846decec26a8222062c441d197489 . "./code.cg2js/cg-dev.fasl")
(#x76d7ada241ccbd0584cca00a0ba1a3b2 . "./code.cg2js/cg-dialog-item.fasl")
(#xe6b77c0fefad46e1d6212618aa45e17d . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xc4957cf8972d2d3976dfb5d991ee4ab0 . "./code.cg2js/cg-directory-dialog.fasl")
(#x1dbff503f884a2561efc1812cabcf231 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xdd66c3bc69144323ef69d65fa5cc182e . "./code.cg2js/cg-drag-and-drop.fasl")
(#x4c0067cc43ea19f2981f2178284e8615 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xca8b5038472e5129cdb8b1d04fe5f9da . "./code.cg2js/cg-drawable.fasl")
(#xf07d5d53c53c0ae53b1ab81c301c419a . "./code.cg2js/cg-dropping-outline.fasl")
(#x39da6d89bea01c6f2434f441438027ca . "./code.cg2js/cg-edit-in-place.fasl")
(#x50b63e46da19b19e4ea015b2ff412db7 . "./code.cg2js/cg-editable-text.fasl")
(#x8e0a366bdb88c19477ef57e6f0ddf936 . "./code.cg2js/cg-file-dialog.fasl")
(#x1664f4f7be7248dbc35bfcebfb3c8c38 . "./code.cg2js/cg-fill-texture.fasl")
(#xc67d0aada5e4f5ea98bf92b84291d6f2 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x70a8ac63ee70f526f44b82666017c018 . "./code.cg2js/cg-font-dialog.fasl")
(#x172e71f9af9c63f800f8a73034928111 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x19d3f23054fd2ed1201dd731b5955463 . "./code.cg2js/cg-get-pixmap.fasl")
(#xbdd21f6277d16a9a716ea164911e099b . "./code.cg2js/cg-get-position.fasl")
(#x56b14c4bcf7af9997e3d7205c4699d0e . "./code.cg2js/cg-graphics-context.fasl")
(#xca39b1d768cb4de0e68037f015feb201 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x0e01ef894143ea03ef61b585e0decc4f . "./code.cg2js/cg-grid-widget.fasl")
(#x9cac2a24f9be5d08e4692d6f09d72fd5 . "./code.cg2js/cg-group-box.fasl")
(#x9710cf8a377caa1bab2fb7beb8d22768 . "./code.cg2js/cg-header-control.fasl")
(#x59ecd1f1c21a295ab56f5c36d8ff5cff . "./code.cg2js/cg-hotspot.fasl")
(#xef7ed51722145498f55427a9296dd5bd . "./code.cg2js/cg-html-dialog.fasl")
(#x17be0d92adb96d5dc78b22e4b0902c33 . "./code.cg2js/cg-html-widget.fasl")
(#x71006324bf1eb8814164097a3d3057c5 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x128cd1512bf585e79024f167ca7e4078 . "./code.cg2js/cg-icon.fasl")
(#x8a9ab771b653ccd97cc354b9a0d267c3 . "./code.cg2js/cg-ie.fasl")
(#x0823bee5a8caeb3c9e8dc6b0c10d495a . "./code.cg2js/cg-intersect-posbox.fasl")
(#x287afcb2791b262a6155b24759046cc2 . "./code.cg2js/cg-intersect.fasl")
(#xd2b58bd63ee0a74fd55c027c501b9893 . "./code.cg2js/cg-item-list.fasl")
(#x8a3da2cd8d49525059cab8cdf1b0f1c1 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x9d766600c250e51fa9e8d00ca03c727d . "./code.cg2js/cg-lamp.fasl")
(#xfca3898fcab95ec6ca476406a8b5cde0 . "./code.cg2js/cg-layout.fasl")
(#xac9fdcc14049450d8f7c9993011520fb . "./code.cg2js/cg-lettered-menu.fasl")
(#x6abcbbbba462c1ed2593bd0629f69841 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x172401beadaf7ac880fca3e8a609a026 . "./code.cg2js/cg-lisp-text.fasl")
(#x5c453b41f33a05b2dd32fe5e1d1ef962 . "./code.cg2js/cg-lisp-widget.fasl")
(#x3a21499dd161bf6af0ec305d50c0e7ba . "./code.cg2js/cg-list-view.fasl")
(#xb620a762c56bc7bf116c3c62f6c64d5b . "./code.cg2js/cg-mci.fasl")
(#x149422aba40adb47eecf4c298ae0e10b . "./code.cg2js/cg-menu-tooltip.fasl")
(#xea5617be981bdd6bda0c386d2b07d329 . "./code.cg2js/cg-menu.fasl")
(#x71ac9772105cbda45dc569a82b977d13 . "./code.cg2js/cg-message-dialog.fasl")
(#x4cea603b2d2cc290df21ec56fa3c33e5 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x4229c07defa10c9f560dfc38e057d86a . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x75329f04c5bbe0601af366437c06759f . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x534058b2587ae517d1999b188ade4b8c . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x2b1b9e4c4b9601f4736f653bcfda6146 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xa7df5a67ea5152e465c2444eb4cfd55c . "./code.cg2js/cg-nodes.fasl")
(#x2de475a7fc125fde855252d9494a13f0 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xaf58524adfe61d43522b0d27a7328dcb . "./code.cg2js/cg-object-editor.fasl")
(#x09975b868bd0bb2b86dfb20517018d4e . "./code.cg2js/cg-ocx.fasl")
(#xf8d2d97a706592de9d7c1cf1e97b3dce . "./code.cg2js/cg-os-widget.fasl")
(#x679a318bc78a1a46fbf92d1854d58e11 . "./code.cg2js/cg-os-window.fasl")
(#x1ff90933cf633ff539e50e77507e0f18 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xa33a3d0caea070d346eec3f7ef3f87a3 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x0ab272f48032b2bf6525f5c0a397795d . "./code.cg2js/cg-outline.fasl")
(#xb776ce85bc2a840c2302e97993fec8d2 . "./code.cg2js/cg-palette.fasl")
(#xa57ed91f2f578f63f2139ec9f39c3ad1 . "./code.cg2js/cg-paren-matching.fasl")
(#x1ff259418b69739cbeea2ee464293b2f . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xff74412c167e9ef3c235e372c627c969 . "./code.cg2js/cg-picture-widget.fasl")
(#x50e30bd79b8ee007b8848b4afd05dda0 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x2789bdc84038ec224a2cf1f423041824 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xc0d8208642b9245966b0190f863dcf8e . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xa6919d4747504a5fb07cd8f67744b1d3 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xb25169d3a212b1922995a0fda64b71b3 . "./code.cg2js/cg-pixmap.fasl")
(#xd87f7a2a18b7118956cbdeacdecfa65e . "./code.cg2js/cg-player.fasl")
(#x406681b1665e97f20f01d79a1c9c1973 . "./code.cg2js/cg-plot-widget.fasl")
(#x87d1b2c64a469d10e653a2b8352d459d . "./code.cg2js/cg-printing.fasl")
(#xfa72511ae5e12a329c51fd99797f46c1 . "./code.cg2js/cg-progress-indicator.fasl")
(#x11673c805544de7105332a34d6358314 . "./code.cg2js/cg-project-window.fasl")
(#x0cae31140de4c89b762221258eddf9f9 . "./code.cg2js/cg-property.fasl")
(#xa98c77a55a52a584bedecdaa715c5624 . "./code.cg2js/cg-radio-button.fasl")
(#x47dea190373f4b736fe23df5113a90b5 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x7d06c4d5c12859f4417f15639aca0d88 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x0371bff4baa25038d75866c16d6820ba . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x72e620c1e899686050aa844223ab193d . "./code.cg2js/cg-rich-edit.fasl")
(#x9505df170a425ea02d3e659c9eca369c . "./code.cg2js/cg-sample-file-menu.fasl")
(#x233a644ce19bbe9a951eb00fd4431a3a . "./code.cg2js/cg-scaling-stream.fasl")
(#x52ff7e8a56c74c9c1ddf95a7ad27cbf3 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x8239d39f159f58f1f99c1d800002d814 . "./code.cg2js/cg-scroll-bar.fasl")
(#x4ee3d985e4b87dc88e858777e1b7794b . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xe09c9d67139f145df23390be753ae783 . "./code.cg2js/cg-selected-object.fasl")
(#x17627bf2c20f7322c0bd701746d61cb1 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xdf290dbbdda983842666b48849e7f68c . "./code.cg2js/cg-split-bar.fasl")
(#x2eefddbae871b304664df121cd93d36a . "./code.cg2js/cg-static-text.fasl")
(#x9de97b777eb7f07bc7945443cada0cb9 . "./code.cg2js/cg-status-bar.fasl")
(#x4be5a56f2c25bbfde922b01489a19f7e . "./code.cg2js/cg-string-dialog.fasl")
(#x64533c2fb846360c7d4b3685ebde60df . "./code.cg2js/cg-tab-control.fasl")
(#xf23c83dccd4c9fe4aefd33725b452d49 . "./code.cg2js/cg-template-string.fasl")
(#xbfc7bc6b847e65765f30cdeba906b93a . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x4752abe8dec2e3f1be9a0c7602468032 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x529362dc1427a4f6ecdc7a0ca94b9e89 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x7c6eb5d7e748c743abc6af2bd1efbd1b . "./code.cg2js/cg-text-or-combo.fasl")
(#xcce1b13608994f22eb886eabd498e7b7 . "./code.cg2js/cg-text-widget.fasl")
(#xb593264b2ae84d3c6a33ed63ea862c27 . "./code.cg2js/cg-timer.fasl")
(#x352c23938e6bd541d0521002370a77ed . "./code.cg2js/cg-toggling-widget.fasl")
(#x6dc2eb5d967030dc48c898b31f6326df . "./code.cg2js/cg-toolbar.fasl")
(#xf5888211fd48f90ff83d9e1175a07489 . "./code.cg2js/cg-tooltip.fasl")
(#xe7281364b2f5f633a6dedbe3b669ff9c . "./code.cg2js/cg-touch.fasl")
(#x34bc6c013aed5bc40361e9d75210c92d . "./code.cg2js/cg-trackbar.fasl")
(#xa7d937d51f0d93e7531623e17b1d0e9e . "./code.cg2js/cg-tray.fasl")
(#x8159212fcfa3c4e1959ed0ff24260be8 . "./code.cg2js/cg-up-down-control.fasl")
(#xd3e24b55b4161e81b69756c624d2b7bc . "./code.cg2js/cg-utility-dialog.fasl")
(#x4e19518ec7dcaccbb24a953ef9bd9bb5 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x9a32fa6d092222859a1a0cdbee1a60d0 . "./code.cg2js/cg-web-browser.fasl")
(#x9d14a828494bdeffb19efc194d2d2b55 . "./code.cg2js/cg-wrap-string.fasl")
(#xe132321abbf72468fc9be49039c85ccb . "./code.cg2js/cg-yes-no-list.fasl")
(#xfd52243425b615ee5f88142aa39bd777 . "./code.cg2js/cg.fasl")
(#xcb44167ad34b266fd300fd99bee678ee . "./code.cg2js/cg-yes-no-string.fasl")
(#x29bf2b65b3aeb7ae8fa4200471d9c794 . "./code.idejs/ide-acache.fasl")
(#xb0024dcce418367c4187ed1b0f2081bf . "./code.idejs/ide-apropos.fasl")
(#x2ac83f42df19f40adbcd947376d470a8 . "./code.idejs/ide-base.fasl")
(#x86436dd390532fce44cd7820df48fcab . "./code.idejs/ide-class-browser.fasl")
(#xf64fa6ace1a0fa50535a8210e6696c9c . "./code.idejs/ide-class-grapher.fasl")
(#x84913e2f713ff5d537ec7096faf0a18c . "./code.idejs/ide-clipboard.fasl")
(#xf267937995850196cf21c2beca906af7 . "./code.idejs/ide-dev.fasl")
(#xc74ad82ced81899b5228f4dc84360b2d . "./code.idejs/ide-editor.fasl")
(#xf6aedd009223b746d487f2f9b231e362 . "./code.idejs/ide-find-definitions.fasl")
(#xaff70e7fd190734dd995999c13c53c6b . "./code.idejs/ide-find-in-files.fasl")
(#x2291268a44b5b64d03e8f8feef4b6480 . "./code.idejs/ide-form.fasl")
(#x0b18493dc5313435c1ba41c14992c564 . "./code.idejs/ide-grapher.fasl")
(#x16fca76129f8a0d0f8ad9b74522822d2 . "./code.idejs/ide-inspector.fasl")
(#xd2ab7505dc264cbee592c3bfa926187b . "./code.idejs/ide-intro.fasl")
(#x4b33f2912a6d31909f645889cd6c70a0 . "./code.idejs/ide-keyshort.fasl")
(#xadddbfe159e665c02424d1acb8fbaa34 . "./code.idejs/ide-navigator.fasl")
(#x84a3c04b184dd64e0b1dea9cd4bdcc1c . "./code.idejs/ide-package-list.fasl")
(#xdcf851a158efcb3b16844e20cf2cb71c . "./code.idejs/ide-patch-dialog.fasl")
(#x1e2c85298552befdd57eb442b6193ecb . "./code.idejs/ide-process-browser.fasl")
(#xf8f1cb5b37d873ecc4aca03b0b202589 . "./code.idejs/ide-project.fasl")
(#x2f7e5abb5f36b85c203f182b776611b1 . "./code.idejs/ide-trace.fasl")
(#x47eaa1bcd85f6fbd91ff3d6ba79f6998 . "./code.idejs/ide-window-list.fasl")
(#x454cfbea1f0646b7fa8ec425b15e553d . "./code.idejs/ide.fasl")
(#xde049c9404b4ba293f0cbc6a4817dc96 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
